unit Auto;
(*
  Esimerkki
   1) TImageListin kytst
   2) TPaintBoxin perimisest
   3) TBitMapin kytst (mm. peilaaminen y-akselin suhteen)
   4) Viestin lhettmisest (Event) omistajalle
       - auto lhett aina liikahtaessaan viestin
       - auto lhettn viestin kun se kntyy

   Vesa Lappalainen 23.10.1997
   

*)
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type
  TOnAutoMove  = procedure (sender:TObject;x,y:integer) of object;
  TOnChangeDir = procedure (sender:TObject;dir:integer) of object;

type
  TAuto = class(TPaintBox)
  private
    FVauhti : integer;
    FSuunta : integer;
    FTimer  : TTimer;
    dx,dy   : integer;
    FOnAutoMove : TOnAutoMove;
    FOnChangeDir : TOnChangeDir;
  protected
    FBitName : String;
    FImageList : TImageList;
    procedure Paint; override;
    procedure TimerTimer(Sender: TObject);   virtual;
  public
    constructor Create(AOwner:TComponent); override;
    destructor  Destroy; override;
    procedure SetBitName(s:string); virtual;
    procedure SetSuunta(i:integer); virtual;
    procedure SetVauhti(i:integer); virtual;
  published
    property BitName : string read FBitName write SetBitName;
    property Vauhti : integer read FVauhti write SetVauhti default 0;
    property Suunta : integer read FSuunta write SetSuunta default 1;
    property OnAutoMove : TOnAutoMove read FOnAutoMove write FOnAutoMove;
    property OnChangeDir : TOnChangeDir read FOnChangeDir write FOnChangeDir;
  end;

procedure Register;

implementation

constructor TAuto.Create(AOwner:TComponent);
begin
  inherited;
  FImageList := TImageList.Create(self);
  FTimer := TTimer.Create(self);
  FTimer.Interval := 100;
  FTimer.Enabled := false;
  FTimer.OnTimer := TimerTimer;
  FVauhti := 0;
  FSuunta := 1;
  dy := 0;
  dx := FVauhti;
  FOnAutoMove := NIL;
  FOnChangeDir := NIL;
end;

destructor TAuto.Destroy;
begin
  FTimer.Free;
  FImageList.Free;
  inherited;
end;

procedure TAuto.Paint;
var index : integer;
begin
  index := 0; if ( dx < 0 ) then index := 1;     // Kumpiko kuva pit piirt
  FImageList.Draw(Canvas,0,0,index);
  inherited;
end;

procedure TAuto.SetSuunta(i:integer);
begin
  if ( i <> FSuunta ) then if Assigned(FOnChangeDir) then FOnChangeDir(self,i);
  FSuunta := i;
  dx := Suunta * Vauhti;
  Paint;                                         // Varmistetaan ett piirretn
end;

procedure TAuto.SetVauhti(i:integer);
begin
  FVauhti := i;
  dx := Suunta * Vauhti;
  FTimer.Enabled := ( dx <> 0 );                 // Timeri kyntiin jos liikkuu
end;

procedure TAuto.SetBitName(s:string);
// Kun bittikartan tiedoston nimi vaihdetaan, pyyhitn vanhat kuvat pois ja
// luodaan uudet kuvat.  Vasemmalle menev kuva saadaan peilaamalla oikealle
// menevt kuva y-akselin suhteen.
var rect,rect2 : TRect;
    lbmp,rbmp :  TBitmap; // Vasemmalle ja oikealle menevt kuvat
begin
  FImageList.Clear;                              // Pyyhitn vanhat kuvat pois
  if ( s = '' ) then begin Paint; exit; end;     // Jos ei nime, ei kuvia
  rbmp := TBitmap.Create;                        // Luodaan tilap. bittikartat
  lbmp := TBitmap.Create;                        // Oik. ja avsemmalle

  FBitName := s;                                 // Muistetaan kuvan nimi
  rbmp.LoadFromFile(s);                          // Luetaan Oik. kuva tied.

  lbmp.Assign(rbmp);                             // Tehdn vas. identtinen
  rect.Left   := 0;                              // Apusuorakaiteet kop.
  rect.Top    := 0;
  rect.Right  := rbmp.Width;
  rect.Bottom := rbmp.Height;
  rect2       := rect;                           // rect2 on rectin peilikuva
  rect2.Right := 0;
  rect2.Left  := rect.Right;
  lbmp.Canvas.CopyRect(rect2,rbmp.Canvas, rect); // lbmp on rbmp:n peilikuva
//  FRightBitmap.Canvas.StretchDraw(rect2,FLeftBitmap); // Toimisi mys

  Width  := rbmp.Width;                          // Oma koko bitmapin kokoiseksi
  Height := rbmp.Height;
  FImageList.Width  := Width;                    // Samoin kuvalistan
  FImageList.Height := Height;
  FImageList.AddMasked(rbmp,rbmp.TransparentColor); // Kuvalistan paik. 0 rbmp
  FImageList.AddMasked(lbmp,lbmp.TransparentColor); //                  1 lbmp
  Paint;
  lbmp.Free;                                     // Poistetaan tilap. kuvat
  rbmp.Free;
end;

procedure TAuto.TimerTimer(Sender: TObject);
begin
  if ( top + dy < 0 ) then dy := abs(dy);
  if ( top + dy >= Parent.ClientHeight-Height ) then dy := -abs(dy);
  if ( left + dx < 0 ) then suunta := 1;
  if ( left + dx >= Parent.ClientWidth-Width ) then suunta := -1;
  top  := top + dy;
  left := left + dx;
  if Assigned(FOnAutoMove) then FOnAutoMove(self,left,top);
end;


procedure Register;
begin
  RegisterComponents('GKO97', [TAuto]);
end;

end.
